<?php
/* --------------------------------------------------------------
   ManufacturerWriteService.inc.php 2017-08-11
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ManufacturerWriteService
 *
 * @category   System
 * @package    Manufacturer
 */
class ManufacturerWriteService implements ManufacturerWriteServiceInterface
{
	/**
	 * @var \ManufacturerRepositoryInterface
	 */
	private $repository;
	
	
	public function __construct(ManufacturerRepositoryInterface $repository)
	{
		$this->repository = $repository;
	}
	
	
	/**
	 * Saves manufacturer entity in database.
	 *
	 * @param \ManufacturerInterface $manufacturer Manufacturer unit to be saved.
	 *
	 * @return $this|\ManufacturerWriteServiceInterface Same instance for chained method calls.
	 */
	public function save(ManufacturerInterface $manufacturer)
	{
		$this->repository->save($manufacturer);
		
		return $this;
	}
	
	
	/**
	 * Deletes manufacturer entity from database.
	 *
	 * @param \ManufacturerInterface $manufacturer Manufacturer unit to be deleted.
	 *
	 * @return $this|\ManufacturerWriteServiceInterface Same instance for chained method calls.
	 */
	public function delete(ManufacturerInterface $manufacturer)
	{
		$this->repository->delete($manufacturer);
		
		return $this;
	}
	
	
	/**
	 * Creates manufacturer entity.
	 *
	 * @return \Manufacturer New manufacturer entity.
	 */
	public function createManufacturer()
	{
		return $this->repository->createManufacturer();
	}
}